@@ -45,22 +45,6 @@ for (pp in 1:p){
 return(auxa)
 })
 
-##################################################
-# One more..
-##################################################
-revvech <- cmpfun(function(x){
-p <- (sqrt(1 + 8 * length(x)) - 1)/2
-if (p != round(p)) stop(" revvech: Input vector not of vech format ")
-out <- matrix(0,p,p)
-ct <- 1
-for (jj in 1:p){
-	out[jj:p,jj] <- x[ct:(ct+p-jj)]
-	ct <- ct + p + 1 - jj
-}
-out[upper.tri(out)] <- out[lower.tri(out)]
-return(out)
-})
-
 ##################################################
 # Regressor matrix for forecasting
 ##################################################
@@ -265,7 +249,7 @@ getmix <- cmpfun(function(){
 ##################################################
 # Estimate Primiceri BVAR with SV and TVP 
 ##################################################
-bvar.sv.tvp <- cmpfun(function(Y, p = 1, tau = 40, nf = 10, pdrift = TRUE, nrep = 50000, nburn = 5000, thinfac = 10, itprint = 10000, k_B = 4, k_A = 4, k_sig = 1, k_Q = 0.01, k_S = 0.1, k_W = 0.01, pQ = NULL, pW = NULL, pS = NULL){
+bvar.sv.tvp <- cmpfun(function(Y, p = 1, tau = 40, nf = 10, pdrift = TRUE, nrep = 50000, nburn = 5000, thinfac = 10, itprint = 10000, save.parameters = TRUE, k_B = 4, k_A = 4, k_sig = 1, k_Q = 0.01, k_S = 0.1, k_W = 0.01, pQ = NULL, pW = NULL, pS = NULL){
 
   # Input checks
 
@@ -362,8 +346,18 @@ bvar.sv.tvp <- cmpfun(function(Y, p = 1, tau = 40, nf = 10, pdrift = TRUE, nrep
   statedraw <- 5*matrix(1,t,M)
   Zs <- matrix(1,t,1) %x% diag(M)
   prw <- matrix(0,7,1)
-  fc.y <- fc.m <- array(0,c(M,nf,nrep))
-  fc.v <- array(0,c(M*(M+1)*0.5,nf,nrep))
+  # new stuff (March 29, 2015)
+  nrep2 <- nrep/thinfac # effective number of draws
+  # re-design forecast output (save only thinned draws)
+  fc.y <- fc.m <- array(0,c(M,nf,nrep2))
+  fc.v <- array(0,c(M*(M+1)*0.5,nf,nrep2))  
+  if (save.parameters == TRUE){
+    # arrays for parameter draws
+    Bt.alldraws <- array(0, c(K,t,nrep2))
+    Ht.alldraws <- Sigt.alldraws <- At.alldraws <- array(0, c(M,M*t,nrep2))
+  } else {
+    Bt.alldraws <- Ht.alldraws <- NULL
+  }
   
   # Storage matrices for (running) posterior means
   
@@ -388,9 +382,12 @@ bvar.sv.tvp <- cmpfun(function(Y, p = 1, tau = 40, nf = 10, pdrift = TRUE, nrep
   
   # Gibbs sampler
   
-  prints <- seq(from=0,to=(nrep+nburn),by=itprint)
+  prints <- seq(from=nburn,to=(nrep+nburn),by=itprint)
   print(paste(Sys.time(),"-- now starting MCMC"))
   
+  # auxiliary counter for saved draws
+  aux.ct <- 0
+  
   for (irep in 1:(nrep+nburn)){
     
     if (irep %in% prints) print(paste(Sys.time(),"-- now at iteration",irep))
@@ -453,7 +450,8 @@ bvar.sv.tvp <- cmpfun(function(Y, p = 1, tau = 40, nf = 10, pdrift = TRUE, nrep
     # Save post-burnin draws and forecasts
     ##############################################################################################################
     
-    if (irep > nburn){
+    if ( (irep > nburn) & ( (irep-nburn) %% thinfac == 0) ){
+	  aux.ct <- aux.ct + 1 # counter for saved draws
       
       Bt_postmean <- Bt_postmean + Btdraw 
       At_postmean <- At_postmean + Atdraw 
@@ -481,9 +479,9 @@ bvar.sv.tvp <- cmpfun(function(Y, p = 1, tau = 40, nf = 10, pdrift = TRUE, nrep
       # Forecasts (with parameter drift)
       
       tempfc <- getfcsts(lastcol(Btdraw), lastcol(Atdraw), lastcol(Sigtdraw), Qdraw, Sdraw, Wdraw, t(y), nf, p)   
-	  fc.m[,,irep-nburn] <- tempfc$mean
-	  fc.v[,,irep-nburn] <- tempfc$variance
-	  fc.y[,,irep-nburn] <- tempfc$draw
+	  fc.m[,,aux.ct] <- tempfc$mean
+	  fc.v[,,aux.ct] <- tempfc$variance
+	  fc.y[,,aux.ct] <- tempfc$draw
       
 	  } else {
 	  
@@ -496,12 +494,23 @@ bvar.sv.tvp <- cmpfun(function(Y, p = 1, tau = 40, nf = 10, pdrift = TRUE, nrep
 
       for (hhh in 1:nf){
         auxl <- varfcst(parmat, tmpvar, fcdat, hhh)
-        fc.m[,hhh,irep-nburn] <- auxl$mean
-        fc.v[,hhh,irep-nburn] <- vechC(auxl$variance)
-		fc.y[,hhh,irep-nburn] <- mvndrawC(auxl$mean, auxl$variance)
+        fc.m[,hhh,aux.ct] <- auxl$mean
+        fc.v[,hhh,aux.ct] <- vechC(auxl$variance)
+		fc.y[,hhh,aux.ct] <- mvndrawC(auxl$mean, auxl$variance)
       }      
 	  
 	  }
+	  
+	  # Save parameter draws
+	
+	  if (save.parameters == TRUE){
+	
+	    Bt.alldraws[,,aux.ct] <- Btdraw
+		Ht.alldraws[,,aux.ct] <- t(Ht)
+		Sigt.alldraws[,,aux.ct] <- Sigtdraw
+		At.alldraws[,,aux.ct] <- t(capAt)
+	  
+	  }
       
     }
     
@@ -509,15 +518,15 @@ bvar.sv.tvp <- cmpfun(function(Y, p = 1, tau = 40, nf = 10, pdrift = TRUE, nrep
   
   # Final steps
   
-  Bt_postmean <- Bt_postmean / nrep
-  At_postmean <- At_postmean / nrep
-  Sigt_postmean <- Sigt_postmean / nrep
-  Ht_postmean <- Ht_postmean / nrep
-  Qmean <- Qmean / nrep
-  Smean <- Smean / nrep
-  Wmean <- Wmean / nrep
-  sigmean <- sigmean / nrep
-  cormean <- cormean / nrep
+  Bt_postmean <- Bt_postmean / nrep2
+  At_postmean <- At_postmean / nrep2
+  Sigt_postmean <- Sigt_postmean / nrep2
+  Ht_postmean <- Ht_postmean / nrep2
+  Qmean <- Qmean / nrep2
+  Smean <- Smean / nrep2
+  Wmean <- Wmean / nrep2
+  sigmean <- sigmean / nrep2
+  cormean <- cormean / nrep2
   
   # Prepare outputs
   
@@ -529,42 +538,224 @@ bvar.sv.tvp <- cmpfun(function(Y, p = 1, tau = 40, nf = 10, pdrift = TRUE, nrep
 	h.out[,,jj] <- Ht_postmean[((jj-1)*M+1):(jj*M),]
   }  
   
-  # Thinning sequence for MCMC draws
-  
-  tsq <- round(seq(from = 0, by = thinfac, to = nrep))[-1]
-  
   # Return list of outputs
   
-  return(list(Beta.postmean = beta.out, H.postmean = h.out, Q.postmean = Qmean, S.postmean = Smean, W.postmean = Wmean, fc.mdraws = fc.m[,,tsq], fc.vdraws = fc.v[,,tsq], fc.ydraws = fc.y[,,tsq]))
+  return(list(Beta.postmean = beta.out, H.postmean = h.out, Q.postmean = Qmean, S.postmean = Smean, W.postmean = Wmean, fc.mdraws = fc.m, fc.vdraws = fc.v, fc.ydraws = fc.y, Beta.draws = Bt.alldraws, 
+              H.draws = Ht.alldraws, logs2.draws = Sigt.alldraws, A.draws = At.alldraws, M = M, p = p))
   
 })
 
-predictive.density <- function(fit, v = 1, h = 1, cdf = FALSE){
+# Helper function to revert vech operator (needed to spot variance elements from VCV matrix)
+vels <- cmpfun(function(n){
+  aux <- matrix(0, n, n)
+  aux[lower.tri(aux, diag = TRUE)] <- 1:(0.5*n*(n+1))
+  return(diag(aux)) 
+})
+
+predictive.density <- cmpfun(function(fit, v = 1, h = 1, cdf = FALSE){
   # Retrieve number of variables/horizons from fit
   nv <- length(fit$fc.ydraws[, 1, 1])
   nh <- length(fit$fc.ydraws[1, , 1])
   # Input check
   if (v > nv | h > nh | v != round(v) | h != round(h))  stop("Please choose appropriate variable and horizon indices")
-  # Locate variance elements in fc.vdraws
-  els <- 1
-  for (z in 2:nv) els <- c(els, els[length(els)] + nv - z + 2)
+  # Locate variance 
+  v.ind <- vels(nv)[v]
   # Get means and standard deviations
   m <- fit$fc.mdraws[v, h, ]
-  s <- sqrt(fit$fc.vdraws[els[v], h, ])
+  s <- sqrt(fit$fc.vdraws[v.ind, h, ])
   # Finally: Return forecast pdf/cdf
   if (cdf == FALSE){
     return(function(q) sapply(q, function(o) mean(dnorm(o, mean = m, sd = s))))
   } else {
     return(function(q) sapply(q, function(o) mean(pnorm(o, mean = m, sd = s))))
   }
-}
+})
 
-predictive.draws <- function(fit, v = 1, h = 1){
+predictive.draws <- cmpfun(function(fit, v = 1, h = 1){
   # Retrieve number of variables/horizons from fit
   nv <- length(fit$fc.ydraws[, 1, 1])
   nh <- length(fit$fc.ydraws[1, , 1])
   # Input check
   if (v > nv | h > nh | v != round(v) | h != round(h))  stop("Please choose appropriate variable and horizon indices")
-  # Return draws
-  return(fit$fc.ydraws[v, h, ])  
-}
\ No newline at end of file
+  # Draws for y
+  y <- fit$fc.ydraws[v, h, ]
+  # Draws for m
+  m <- fit$fc.mdraws[v, h, ]
+  # Draws for v
+  v <- fit$fc.vdraws[vels(nv)[v], h, ] 
+  # Return draws in list
+  return(list(y = y, m = m, v = v))  
+})
+
+parameter.draws <- cmpfun(function(fit, type = "lag1", row = 1, col = 1){
+  # Input checks
+  if (is.null(fit$Beta.draws)) stop("Parameter draws in fit have not been saved -- please run again, using the option `save.parameters = TRUE'.")
+  nms <- c("intercept", paste0("lag", 1:fit$p), "vcv")
+  if (!type %in% nms) stop(paste("Type unknown -- please enter one of the following:", paste(nms, collapse = "; ")))
+  if (type == "intercept"){
+    if (!row %in% 1:fit$M) stop(paste("Invalid row selected -- please enter one of the following:", paste(1:fit$M, collapse = "; ")))
+  } else if (type %in% c(paste0("lag", 1:fit$p), "vcv")){
+	if (!row %in% 1:fit$M | !col %in% 1:fit$M) stop(paste("Invalid row or col selected -- please enter one of the following:", paste(1:fit$M, collapse = "; ")))
+  }
+  
+  # Intermediate stuff
+  Beta.draws <- fit$Beta.draws  
+  t <- dim(Beta.draws)[2]
+  
+  # Get parameters 
+  if (type == "vcv"){
+    aux.seq <- seq(from = col, by = fit$M, length.out = t)
+    out <- t(fit$H.draws[row, aux.seq, ])
+  } else {
+    if (type == "intercept"){
+	  # Parameter output
+	  out <- t(Beta.draws[row, , ])
+	  # Needed for checking below
+      aux.col <- 1
+	} else if (type %in% paste0("lag", 1:fit$p)){
+	  # Lag order
+	  lo <- as.numeric(substr(type, 4, nchar(type)))
+	  # Find position in storage output
+	  ind <- fit$M + (lo - 1)*(fit$M^2) + (row - 1)*fit$M + col
+	  # Get draws
+	  out <- t(Beta.draws[ind, , ])
+	  # Needed for checking below
+	  aux.col <- 1 + (lo-1)*fit$M + col
+	}
+    # Check (compare to slower beta.reshape function, for one randomly chosen MCMC draw and time period)
+	aux.ind <- sample.int(dim(Beta.draws)[3], 1)
+	aux.t <- sample.int(dim(Beta.draws)[2], 1)
+	if (out[aux.ind, aux.t] != beta.reshape(Beta.draws[, aux.t, aux.ind], fit$M, fit$p)[row, aux.col]) stop("Wrong output...") 
+  }
+  # Message
+  if (type == "intercept"){
+    message(paste("Element", row, "of intercept vector. Format: MCMC draws in rows, time in columns."))
+  } else {
+    message(paste0("Element [", row, ",", col, "] of ", type, " matrix. Format: MCMC draws in rows, time in columns."))
+  }
+  return(out)
+})
+
+
+
+ARtoMA <- cmpfun(function(A, nhor){
+  # Infer dimensions from A parameters
+  M <- nrow(A)
+  p <- ncol(A)/M
+  Phi <- matrix(0, M, M*nhor) 
+  # See recursive formula for MA (see e.g. page 28 in http://www.jmulti.de/download/help/var.pdf)
+  for (s in 1:nhor){
+    tmp <- matrix(0, M, M)
+	for (j in 1:s){
+	  if (s == j){
+	    aux <- diag(M)
+	  } else {
+	    aux <- Phi[, ((s-j-1)*M+1):((s-j)*M)]
+	  }
+	  if (j <= p) tmp <- tmp + aux %*% A[,((j-1)*M+1):(j*M)]
+	}
+	Phi[,((s-1)*M+1):(s*M)] <- tmp
+  }  
+  Phi  
+})
+
+matmult <- cmpfun(function(mat, mult){
+  if (mult == 0){
+    out <- diag(nrow(mat))
+  } else if (mult == 1){
+    out <- mat
+  } else {
+    out <- mat
+    for (c in 1:(mult-1)) out <- out %*% mat    
+  }
+  return(out)
+})
+
+IRFmats <- cmpfun(function(A, H.chol = NULL, nhor, orthogonal = TRUE){
+  # Dimensions
+  M <- nrow(A) # nr of variables
+  
+  # AR matrices
+  p <- ncol(A)/M
+      
+  # Construct companion form matrices
+  
+  if (p > 1){
+      
+    # VAR matrices
+    Ac <- matrix(0, M*p, M*p)
+    Ac[1:M, ] <- A
+    Ac[-(1:M), 1:(M*(p-1))] <- diag(M*(p-1))
+    
+  } else {
+  
+    Ac <- A
+  
+  }
+  
+  # Matrix with impulse responses
+  Phi <- matrix(0, M, M*(nhor+1)) 
+  for (s in 0:nhor){
+    aux <- matmult(Ac, s)[1:M, 1:M] 
+	if (!is.null(H.chol)) aux <- aux %*% H.chol
+	Phi[,(s*M+1):((s+1)*M)] <- aux
+  } 
+  Phi
+  
+})
+
+impulse.responses <- cmpfun(function(fit, impulse.variable = 1, response.variable = 2, t = NULL, nhor = 20, scenario = 2, draw.plot = TRUE){
+
+  # Get coefficient draws from fit
+  Beta.draws <- fit$Beta.draws
+  nd <- dim(Beta.draws)[3]
+  if (is.null(t)) t <- dim(Beta.draws)[2]
+  out <- matrix(0, nd, nhor + 1)
+  M <- fit$M
+  p <- fit$p
+  
+  # Get relevant stuff for VCV matrix
+  H.sel <- fit$H.draws[,((t-1)*M+1):(t*M),]
+  A.sel <- fit$A.draws[,((t-1)*M+1):(t*M),]
+  
+  if (scenario == 3){
+    # Compute mean of sigma (over both time and MCMC draws, as in Primiceri's code)
+    sig <- apply(exp(0.5*fit$logs2.draws), 1, mean)
+    sig <- diag(sig)   
+  } else {
+    sig <- NULL
+  }
+    
+  # Compute IR for each MC iteration
+  for (j in 1:nd){
+  
+    # Cholesky of VCV matrix, depending on specification
+    if (scenario == 1){ # No orthogonalization at all
+	  H.chol <- NULL
+	} else if (scenario == 2){ # Standard orthogonalization
+	  H.chol <- t(chol(H.sel[,,j]))
+	} else if (scenario == 3){ # Orthogonalization as in Primiceri
+	  H.chol <- t(solve(A.sel[,,j])) %*% sig
+	}
+
+	# Compute Impulse Responses
+    aux <- IRFmats(A = beta.reshape(Beta.draws[,t,j], M, p)[,-1], H.chol = H.chol, nhor = nhor)
+	aux <- aux[response.variable, seq(from = impulse.variable, by = M, length = nhor + 1)]  	
+	
+  	out[j,] <- aux
+  }
+  # Make plot
+  if (draw.plot){
+    pdat <- t(apply(out[,-1], 2, function(z) quantile(z, c(0.05, 0.25, 0.5, 0.75, 0.95))))
+    xax <- 1:nhor
+	matplot(x = xax, y = pdat, type = "n", ylab = "", xlab = "Horizon", bty = "n", xlim = c(1, nhor))
+	polygon(c(xax, rev(xax)), c(pdat[,5], rev(pdat[,4])), col = "grey60", border = NA)
+	polygon(c(xax, rev(xax)), c(pdat[,4], rev(pdat[,3])), col = "grey30", border = NA)
+	polygon(c(xax, rev(xax)), c(pdat[,3], rev(pdat[,2])), col = "grey30", border = NA)
+	polygon(c(xax, rev(xax)), c(pdat[,2], rev(pdat[,1])), col = "grey60", border = NA)
+	lines(x = xax, y = pdat[,3], type = "l", col = 1, lwd = 2.5)
+	abline(h = 0, lty = 2)
+  }
+  list(contemporaneous = out[,1], irf = out[,-1])  
+})
+  
\ No newline at end of file
